/*
 * HttSquash, http server scanner, banner grabber and data retriever
 * Copyright (C) 2008  Acri Emanuele <crossbower@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <unistd.h>
#include <sys/wait.h>
#include <signal.h>
#include <ctype.h>

#include "httsquash.h"
#include "argparser.h"
#include "fingerprint.h"

/*
 * Catch alarm signal (and others)
 * and exit quietly...
 */
void catch_alarm (int sig)
{
    exit(0);
}

/*
 * Scan loop that assembles current ip address
 * and exec a callback function
 */
void scanloop(struct iprange *ipr, void *p)
{
    void (*fp) (struct iprange *) = NULL;
    fp = p;

    while(ipr->curr <= ipr->end) {
    
        // current address
        if(ipr->ipver==VER_IPV4)
            sprintf(ipr->addr, "%s.%u", ipr->block, ipr->curr );
        else
            sprintf(ipr->addr, "%s:%x", ipr->block, ipr->curr );

        // callback function
        (*fp) (ipr);

        ipr->curr++;
    }
}

/*
 * Function called by scanloop()
 * Gets the ip address and retrieve informations from it
 */
void scanstep(struct iprange *ipr)
{
    int sock;
    char *buffer, *payload, *payend;
    pid_t pid;
    
    count++;

    //fork
    pid=fork();

    //child (that scan...)
    if(pid==0) {

        // Establish a handler for SIGALRM signals
        signal (SIGALRM, catch_alarm);
        signal (SIGTERM, catch_alarm);
        signal (SIGKILL, catch_alarm);
        
        alarm(ipr->time); //timer
        
        sock=opensock(ipr->addr, ipr->port); //open socket
        if(sock < 0) {
            //fprintf(stderr, "\nopensock returned: %d\n", sock);
            exit(0);
        }
        
        // Experiment...
        //alarm(0);
        
        senddata(sock, ipr->http_request); //send data
        
        if(ipr->verbose)
            buffer = recvdataloop(sock, MAX_DATA*4);
        else
            buffer = recvdata(sock, MAX_DATA);

        closesock(sock); //close socket

        // PRINT HEADERS

        payload = strstr(buffer, "\r\n\r\n"); //isolate http headers
        if(payload!=NULL) *payload = '\0';

        printf("FOUND: %s %d\n", ipr->addr, ipr->port);
        printf("%s\n", buffer);
        puts("");
        
        //if(payload!=NULL) *payload = '\r';
        
        // PRINT DATA
        
        if(ipr->verbose) {
        
            payload += 4;
            
            // chunked data
            if(strstr(payload, "\r\n0\r\n\r\n")!=NULL) {
            
                payload = strstr(payload, "\n"); // isolate html data
                payload ++;
                
                payend = strstr(payload, "\r\n0\r\n\r\n"); // end of html data
                
                if(payend!=NULL) {
                    *payend = '\0';
                }
            }
            
            puts("DATA:");
            printf("%s\n", payload);
            puts("");
        }
        
        // FINGERPRINT
        if(ipr->fingerprint) {
            puts("FINGERPRINT:");
            fingerprint(ipr, buffer);
            puts("");
        }
        
        if(ipr->cookiejar)
            puts("%%\n");

        free(buffer);

        exit(0);
    }

    //dad (that wait...)
    else {
        if(count>=MAX_PROC) {
            for(count; count>0; count--) {
                wait(0);
            }
        }
    }
}

/*
 * Main function
 */
int main(int argc, char **argv)
{
    struct iprange *ipr;

    // Alloc ipr
    ipr = (struct iprange *) malloc(sizeof(struct iprange));
    
    // Clear ipr
    memset(ipr, 0, sizeof(struct iprange));
    
    // Parse options
    parseopt(ipr, argc, argv);

    // Init scan loop
    scanloop(ipr, &scanstep);

    // Wait last childs...
    for(count; count>0; count--) wait(0);

    // Dealloc ipr
    free(ipr);

    return 0;
}

